@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 __copyright__ = """
 Copyright (C) 2009-2017 Andreas Kloeckner
 Copyright (C) 2014-2017 Aaron Meurer
@@ -105,7 +103,8 @@ class SourceLine(urwid.FlowWidget):
             line_prefix = self.line_nr
 
         line_prefix = crnt+bp+line_prefix
-        line_prefix_attr = [("source", 1), ("breakpoint marker", 1)] \
+        line_prefix_attr = [("current line marker", 1),
+                            ("breakpoint marker", 1)] \
                 + line_prefix_attr
 
         # assume rendered width is same as len
@@ -160,163 +159,14 @@ class SourceLine(urwid.FlowWidget):
         return key
 
 
-def format_source(debugger_ui, lines, breakpoints):
-    lineno_format = "%%%dd " % (len(str(len(lines))))
-    try:
-        import pygments  # noqa
-    except ImportError:
-        return [SourceLine(debugger_ui,
-            line.rstrip("\n\r").expandtabs(TABSTOP),
-            lineno_format % (i+1), None,
-            has_breakpoint=i+1 in breakpoints)
-            for i, line in enumerate(lines)]
-    else:
-        from pygments import highlight
-        from pygments.lexers import PythonLexer
-        from pygments.formatter import Formatter
-        import pygments.token as t
-
-        result = []
-        argument_parser = ArgumentParser(t)
-
-        # NOTE: Tokens of the form t.Token.<name> are not native
-        #       Pygments token types; they are user defined token
-        #       types.
-        #
-        #       t.Token is a Pygments token creator object
-        #       (see http://pygments.org/docs/tokens/)
-        #
-        #       The user defined token types get assigned by
-        #       one of several translation operations at the
-        #       beginning of add_snippet().
-        #
-        ATTR_MAP = {  # noqa: N806
-                t.Token: "source",
-                t.Keyword.Namespace: "namespace",
-                t.Token.Argument: "argument",
-                t.Token.Dunder: "dunder",
-                t.Token.Keyword2: "keyword2",
-                t.Keyword: "keyword",
-                t.Literal: "literal",
-                t.Name.Exception: "exception",
-                t.Name.Function: "name",
-                t.Name.Class: "name",
-                t.Name.Builtin: "builtin",
-                t.Name.Builtin.Pseudo: "pseudo",
-                t.Punctuation: "punctuation",
-                t.Operator: "operator",
-                t.String: "string",
-                # XXX: Single and Double don't actually work yet.
-                # See https://bitbucket.org/birkenfeld/pygments-main/issue/685
-                t.String.Double: "doublestring",
-                t.String.Single: "singlestring",
-                t.String.Backtick: "backtick",
-                t.String.Doc: "docstring",
-                t.Comment: "comment",
-                }
-
-        # Token translation table. Maps token types and their
-        # associated strings to new token types.
-        ATTR_TRANSLATE = {  # noqa: N806
-                t.Keyword: {
-                    "class": t.Token.Keyword2,
-                    "def": t.Token.Keyword2,
-                    "exec": t.Token.Keyword2,
-                    "lambda": t.Token.Keyword2,
-                    "print": t.Token.Keyword2,
-                    },
-                t.Operator: {
-                    ".": t.Token,
-                    },
-                t.Name.Builtin.Pseudo: {
-                    "self": t.Token,
-                    },
-                t.Name.Builtin: {
-                    "object": t.Name.Class,
-                    },
-                }
-
-        class UrwidFormatter(Formatter):
-            def __init__(subself, **options):  # noqa: N805
-                Formatter.__init__(subself, **options)
-                subself.current_line = ""
-                subself.current_attr = []
-                subself.lineno = 1
-
-            def format(subself, tokensource, outfile):  # noqa: N805
-                def add_snippet(ttype, s):
-                    if not s:
-                        return
-
-                    # Find function arguments. When found, change their
-                    # ttype to t.Token.Argument
-                    new_ttype = argument_parser.parse_token(ttype, s)
-                    if new_ttype:
-                        ttype = new_ttype
-
-                    # Translate tokens
-                    if ttype in ATTR_TRANSLATE:
-                        if s in ATTR_TRANSLATE[ttype]:
-                            ttype = ATTR_TRANSLATE[ttype][s]
-
-                    # Translate dunder method tokens
-                    if ttype == (
-                            t.Name.Function
-                            and s.startswith("__") and s.endswith("__")
-                            ):
-                        ttype = t.Token.Dunder
-
-                    while ttype not in ATTR_MAP:
-                        if ttype.parent is not None:
-                            ttype = ttype.parent
-                        else:
-                            raise RuntimeError(
-                                    "untreated token type: %s" % str(ttype))
-
-                    attr = ATTR_MAP[ttype]
-
-                    subself.current_line += s
-                    subself.current_attr.append((attr, len(s)))
-
-                def shipout_line():
-                    result.append(
-                            SourceLine(debugger_ui,
-                                subself.current_line,
-                                lineno_format % subself.lineno,
-                                subself.current_attr,
-                                has_breakpoint=subself.lineno in breakpoints))
-                    subself.current_line = ""
-                    subself.current_attr = []
-                    subself.lineno += 1
-
-                for ttype, value in tokensource:
-                    while True:
-                        newline_pos = value.find("\n")
-                        if newline_pos == -1:
-                            add_snippet(ttype, value)
-                            break
-                        else:
-                            add_snippet(ttype, value[:newline_pos])
-                            shipout_line()
-                            value = value[newline_pos+1:]
-
-                if subself.current_line:
-                    shipout_line()
-
-        highlight("".join(line.expandtabs(TABSTOP) for line in lines),
-                PythonLexer(stripnl=False), UrwidFormatter())
-
-        return result
-
-
-class ParseState(object):
+class ParseState:
     """States for the ArgumentParser class"""
     idle = 1
     found_function = 2
     found_open_paren = 3
 
 
-class ArgumentParser(object):
+class ArgumentParser:
     """Parse source code tokens and identify function arguments.
 
     This parser implements a state machine which accepts
@@ -339,7 +189,7 @@ class ArgumentParser(object):
     def parse_token(self, token, s):
         """Parse token. Return None or replacement token type"""
         if self.state == ParseState.idle:
-            if token is self.t.Name.Function:
+            if token in (self.t.Name.Function, self.t.Name.Function.Magic):
                 self.state = ParseState.found_function
                 self.paren_level = 0
         elif self.state == ParseState.found_function:
@@ -347,8 +197,7 @@ class ArgumentParser(object):
                 self.state = ParseState.found_open_paren
                 self.paren_level = 1
         else:
-            if ((token is self.t.Name)
-                    or (token is self.t.Name.Builtin.Pseudo and s == "self")):
+            if (token is self.t.Name):
                 return self.t.Token.Argument
             elif token is self.t.Punctuation and s == ")":
                 self.paren_level -= 1
@@ -357,3 +206,162 @@ class ArgumentParser(object):
             if self.paren_level == 0:
                 self.state = ParseState.idle
         return None
+
+
+try:
+    import pygments  # noqa
+except ImportError:
+    def format_source(debugger_ui, lines, breakpoints):
+        lineno_format = "%%%dd " % (len(str(len(lines))))
+        return [
+            SourceLine(
+                debugger_ui,
+                line.rstrip("\n\r").expandtabs(TABSTOP),
+                lineno_format % (i+1),
+                None,
+                has_breakpoint=i+1 in breakpoints,
+            )
+            for i, line in enumerate(lines)
+        ]
+else:
+    from pygments import highlight
+    from pygments.lexers import PythonLexer
+    from pygments.formatter import Formatter
+    import pygments.token as t
+
+    argument_parser = ArgumentParser(t)
+
+    # NOTE: Tokens of the form t.Token.<name> are not native
+    #       Pygments token types; they are user defined token
+    #       types.
+    #
+    #       t.Token is a Pygments token creator object
+    #       (see http://pygments.org/docs/tokens/)
+    #
+    #       The user defined token types get assigned by
+    #       one of several translation operations at the
+    #       beginning of add_snippet().
+    #
+    ATTR_MAP = {  # noqa: N806
+        t.Token: "source",
+        t.Keyword.Namespace: "namespace",
+        t.Token.Argument: "argument",
+        t.Token.Dunder: "dunder",
+        t.Token.Keyword2: "keyword2",
+        t.Keyword: "keyword",
+        t.Literal: "literal",
+        t.Name: "name",
+        t.Name.Exception: "exception",
+        t.Name.Function: "function",
+        t.Name.Function.Magic: "magic",
+        t.Name.Class: "class",
+        t.Name.Builtin: "builtin",
+        t.Name.Builtin.Pseudo: "pseudo",
+        t.Name.Variable.Magic: "magic",
+        t.Punctuation: "punctuation",
+        t.Operator: "operator",
+        t.String: "string",
+        t.String.Double: "doublestring",
+        t.String.Single: "singlestring",
+        t.String.Backtick: "backtick",
+        t.String.Doc: "docstring",
+        t.Comment: "comment",
+    }
+
+    # Token translation table. Maps token types and their
+    # associated strings to new token types.
+    ATTR_TRANSLATE = {  # noqa: N806
+            t.Keyword: {
+                "class": t.Token.Keyword2,
+                "def": t.Token.Keyword2,
+                "exec": t.Token.Keyword2,
+                "lambda": t.Token.Keyword2,
+                "print": t.Token.Keyword2,
+                },
+            t.Operator: {
+                ".": t.Token,
+                },
+            }
+
+    class UrwidFormatter(Formatter):
+        def __init__(self, debugger_ui, lineno_format, breakpoints, **options):
+            Formatter.__init__(self, **options)
+            self.current_line = ""
+            self.current_attr = []
+            self.lineno = 1
+            self.result = []
+            self.debugger_ui = debugger_ui
+            self.lineno_format = lineno_format
+            self.breakpoints = breakpoints
+
+        def add_snippet(self, ttype, s):
+            if not s:
+                return
+
+            # Find function arguments. When found, change their
+            # ttype to t.Token.Argument
+            new_ttype = argument_parser.parse_token(ttype, s)
+            if new_ttype:
+                ttype = new_ttype
+
+            # Translate tokens
+            if ttype in ATTR_TRANSLATE:
+                if s in ATTR_TRANSLATE[ttype]:
+                    ttype = ATTR_TRANSLATE[ttype][s]
+
+            # Translate dunder method tokens
+            # NOTE: leaves "Magic" name tokens alone
+            if (ttype == t.Name.Function
+                    and s.startswith("__")
+                    and s.endswith("__")):
+                ttype = t.Token.Dunder
+
+            while ttype not in ATTR_MAP:
+                if ttype.parent is not None:
+                    ttype = ttype.parent
+                else:
+                    raise RuntimeError(
+                            "untreated token type: %s" % str(ttype))
+
+            attr = ATTR_MAP[ttype]
+
+            self.current_line += s
+            self.current_attr.append((attr, len(s)))
+
+        def shipout_line(self):
+            self.result.append(
+                SourceLine(
+                    self.debugger_ui,
+                    self.current_line,
+                    self.lineno_format % self.lineno,
+                    self.current_attr,
+                    has_breakpoint=self.lineno in self.breakpoints,
+                ))
+            self.current_line = ""
+            self.current_attr = []
+            self.lineno += 1
+
+        def format(self, tokensource, outfile):
+            for ttype, value in tokensource:
+                while True:
+                    newline_pos = value.find("\n")
+                    if newline_pos == -1:
+                        self.add_snippet(ttype, value)
+                        break
+                    else:
+                        self.add_snippet(ttype, value[:newline_pos])
+                        self.shipout_line()
+                        value = value[newline_pos+1:]
+
+            if self.current_line:
+                self.shipout_line()
+
+    def format_source(debugger_ui, lines, breakpoints):
+        lineno_format = "%%%dd " % (len(str(len(lines))))
+        formatter = UrwidFormatter(debugger_ui, lineno_format, breakpoints)
+        highlight(
+            "".join(line.expandtabs(TABSTOP) for line in lines),
+            PythonLexer(stripnl=False),
+            formatter,
+        )
+        return formatter.result
