@@ -16,14 +16,14 @@ class TestNullSourceCodeProvider:
 class TestFileSourceCodeProvider:
     def test_string_file_name(self, mocker):
         mock_debugger = mocker.Mock()
-        mock_debugger.canonic = mocker.Mock(return_value='<string>')
-        provider = FileSourceCodeProvider(mock_debugger, 'test file name')
+        mock_debugger.canonic = mocker.Mock(return_value="<string>")
+        provider = FileSourceCodeProvider(mock_debugger, "test file name")
         result = provider.get_lines(mocker.MagicMock())
         assert len(result) == 1
         assert isinstance(result[0], SourceLine)
 
     def test_get_lines(self, mocker):
-        provider = FileSourceCodeProvider(mocker.Mock(), 'test file name')
+        provider = FileSourceCodeProvider(mocker.Mock(), "test file name")
         result = provider.get_lines(mocker.MagicMock())
         assert len(result) == 1
         assert isinstance(result[0], SourceLine)
@@ -31,7 +31,7 @@ class TestFileSourceCodeProvider:
 
 class TestDirectSourceCodeProvider:
     def test_get_lines(self, mocker):
-        provider = DirectSourceCodeProvider(mocker.Mock(), 'test code')
+        provider = DirectSourceCodeProvider(mocker.Mock(), "test code")
         result = provider.get_lines(mocker.Mock())
         assert len(result) == 1
         assert isinstance(result[0], SourceLine)
